>From b3f96ae452d3d31e72ecd31ad6e9364ab54f37e3 Mon Sep 17 00:00:00 2001
From: Tadas Slotkus <devtadas@gmail.com>
Date: Sat, 13 Aug 2011 18:18:50 +0300
Subject: [carFl mem 07/21] read_flash_to_file: use chunked reads/writes

>From now reading and writing flash uses supplied small buffers

Signed-off-by: Tadas Slotkus <devtadas@gmail.com>
---
 cli_classic.c |    3 ++-
 flash.h       |    2 +-
 flashrom.c    |   32 +++++++++++++++++++-------------
 3 files changed, 22 insertions(+), 15 deletions(-)

diff --git a/cli_classic.c b/cli_classic.c
index be95032..086bd86 100644
--- a/cli_classic.c
+++ b/cli_classic.c
@@ -398,7 +398,8 @@ int cli_classic(int argc, char *argv[])
 			}
 			printf("Please note that forced reads most likely "
 			       "contain garbage.\n");
-			return read_flash_to_file(&flashes[0], filename);
+			uint8_t tmpbuf[BUF_SIZE];
+			return read_flash_to_file(&flashes[0], filename, tmpbuf);
 		}
 		ret = 1;
 		goto out_shutdown;
diff --git a/flash.h b/flash.h
index 09dc65d..7c044bf 100644
--- a/flash.h
+++ b/flash.h
@@ -212,7 +212,7 @@ void map_flash_registers(struct flashchip *flash);
 int read_memmapped(struct flashchip *flash, uint8_t *buf, int start, int len);
 int erase_flash(struct flashchip *flash);
 int probe_flash(int startchip, struct flashchip *fill_flash, int force);
-int read_flash_to_file(struct flashchip *flash, const char *filename);
+int read_flash_to_file(struct flashchip *flash, const char *filename, uint8_t *buf);
 int min(int a, int b);
 int max(int a, int b);
 void tolower_string(char *str);
diff --git a/flashrom.c b/flashrom.c
index 36b5e51..c522c84 100644
--- a/flashrom.c
+++ b/flashrom.c
@@ -1337,16 +1337,19 @@ int write_chunk_to_file(unsigned char *buf, unsigned long start, unsigned int si
 	return 0;
 }
 
-int read_flash_to_file(struct flashchip *flash, const char *filename)
+int read_flash_to_file(struct flashchip *flash, const char *filename, uint8_t *buf)
 {
 	unsigned long size = flash->total_size * 1024;
-	unsigned char *buf = calloc(size, sizeof(char));
-	int ret = 0;
+	int ret = 0, starthere = 0, lenhere = BUF_SIZE;
+	FILE *image;
 
 	msg_cinfo("Reading flash... ");
-	if (!buf) {
-		msg_gerr("Memory allocation failed!\n");
-		msg_cinfo("FAILED.\n");
+	if (!filename) {
+		msg_gerr("No filename specified.\n");
+		return 1;
+	}
+	if ((image = fopen(filename, "wb")) == NULL) {
+		perror(filename);
 		return 1;
 	}
 	if (!flash->read) {
@@ -1354,15 +1357,18 @@ int read_flash_to_file(struct flashchip *flash, const char *filename)
 		ret = 1;
 		goto out_free;
 	}
-	if (flash->read(flash, buf, 0, size)) {
-		msg_cerr("Read operation failed!\n");
-		ret = 1;
-		goto out_free;
+	for (; starthere < size; starthere += lenhere) {
+		if (flash->read(flash, buf, starthere, lenhere)) {
+			msg_cerr("Read operation failed!\n");
+			ret = 1;
+			goto out_free;
+		}
+		if ((ret = write_chunk_to_file(buf, starthere, lenhere, image)))
+			goto out_free;
 	}
 
-	ret = write_buf_to_file(buf, size, filename);
 out_free:
-	free(buf);
+	fclose(image);
 	msg_cinfo("%s.\n", ret ? "FAILED" : "done");
 	return ret;
 }
@@ -1908,7 +1914,7 @@ int doit(struct flashchip *flash, int force, const char *filename, int read_it,
 		flash->unlock(flash);
 
 	if (read_it) {
-		ret = read_flash_to_file(flash, filename);
+		ret = read_flash_to_file(flash, filename, oldcontents);
 		goto out_nofree;
 	}
 
-- 
1.7.6

